function [G] =rootPlotLab02(Kp0,Kp1,Kd0,Kd1,Ki0,Ki1,mIn,BetaIn,GamIn)
% Plots a loci of natural frequencies, and several step responses for 
% a linear model of the propeller arm system.
% The plotted loci are natural frequencies with magnitude > 0.9,
% as Kp, Kd, and Ki vary linearly from Kp0->Kp1, Kd0->Kd1, and Ki0->Ki1 respectively
% The nat freqs associated with Kp1,Kd1,Ki1 are marked with stars in circles.
% The two plotted step responses are for the Kp0,Kd0,Ki0 and Kp1,Kd1,Ki1 cases
% BetaIn and GamIn model the linearized motor command to thrust relation.

% Defaults
close all;
% Auto rescaling around 1.
zoom = false;   % Zooms complex plane

%pSmooth = 0.0;
dT = 1.0e-3;
z = tf([1 0], 1, dT); % Define z as the transform variable
R = 1/z; 
points = 1000;

    function [natFreqs, G, Hc2a] = getNatFreqs(Kp,Kd,Ki,m,Beta,Gam)
        % Gets the transfer function and nat freqs for prop arm model.
        Hw2t = (dT*R)/(1-R); % Rot velocity (w) to theta (angle)
        Ha2w = (dT*R)/(1-R); % Rot accel->rot velocity
        Kpid= Kp + Kd*(1-R^m)/(m*dT) + 0*Ki; % FIX THIS: Add Integral Term.
        Hc2a = Gam; % FIX THIS: cmd -> arm accel, replace instant model.
        Hc2t = Hc2a*Ha2w*Hw2t;  % motor command to angle
        G = feedback(Kpid*Hc2t,1);  % Closed-loop behavior cmd->thrust instant
        natFreqs = pole(G);
    end

rTemplate = getNatFreqs(Kp1,Kd1,Ki1,mIn,BetaIn,GamIn);
disp(rTemplate);
rootMat = zeros(size(rTemplate,1),points);
for i = 1:points
    Kdl = Kd0 + (Kd1-Kd0)*i/points;
    Kpl = Kp0 + (Kp1-Kp0)*i/points;
    Kil = Ki0 + (Ki1-Ki0)*i/points;
    rootMat(:,i) = getNatFreqs(Kpl,Kdl,Kil,mIn,BetaIn,GamIn);
end

figure(1);

[~,G,~] = getNatFreqs(Kp1,Kd1,Ki1,mIn,BetaIn,GamIn);
[V,T] = step(G,5);  % 5 seconds of data
subplot(3,1,1);
plot(T,V);
xlabel('Time in Samples');
title(['Closed-Loop Step Response',...
        ' m=',num2str(mIn),',Kp=',num2str(Kp1),...
        ',Kd=',num2str(Kd1),',Ki=',num2str(Ki1)]);

[~,G,~] = getNatFreqs(Kp0,Kd0,Ki0,mIn,BetaIn,GamIn);
[V,T] = step(G,5);  % 5 seconds of data
subplot(3,1,2);
plot(T,V);
xlabel('Time in Samples');
title(['Closed-Loop Step Response',...
        ' m=',num2str(mIn),',Kp=',num2str(Kp0),...
        ',Kd=',num2str(Kd0),',Ki=',num2str(Ki0)]);

subplot(3,1,3);
if(zoom)
  zoomThresh = 0.95;
else
    zoomThresh = 0;
end
rootsLarge = rootMat(abs(rootMat) > zoomThresh);
    
rootsZ = rootsLarge(imag(rootsLarge) == 0);
rootsUp = rootsLarge(imag(rootsLarge) > 0);
rootsDown = rootsLarge(imag(rootsLarge) < 0);
plot(real(rootsZ),imag(rootsZ),'mo');
hold on;
plot(real(rootsUp),imag(rootsUp),'ro');
plot(real(rootsDown),imag(rootsDown),'go');

if length(rootMat) > 4
   last4 = rootMat(:,end-4:end); 
   last4 = last4(abs(last4) >= zoomThresh);
   plot(real(last4), imag(last4),'k*');
end

maxangle = max(abs(angle(rootsLarge(:))));
ang = linspace(-2*maxangle, 2*maxangle, 10000);

circlepts = cos(ang)+ 1j*sin(ang);
plot(real(circlepts), imag(circlepts), 'b.');
hold off;
title(['1-Circ(Bl),Poles(R+G),maxK poles(Bl*)',...
        ', m=',num2str(mIn),...    
        ', Kp:',num2str(Kp0),'->',num2str(Kp1),...
        ', Kd:',num2str(Kd0),'->',num2str(Kd1),...
        ', Ki:',num2str(Ki0),'->',num2str(Ki1)]);
end


